#include "DrawnObjectBySTL.h"

#pragma warning ( disable : 4996 )

#include "MyUtility.h"

DrawnObjectBySTL::DrawnObjectBySTL(std::string name):DrawnObject(name){
	
	this->singleColor[0] = 1.0f;
	this->singleColor[1] = 1.0f;
	this->singleColor[2] = 1.0f;
	this->isSingleColorEnable = true;
}


DrawnObjectBySTL::DrawnObjectBySTL(GLfloat r,GLfloat g,GLfloat  b,std::string name):DrawnObject(name){
	this->singleColor[0] = r;
	this->singleColor[1] = g;
	this->singleColor[2] = b;
	this->isSingleColorEnable = true;
}


DrawnObjectBySTL::~DrawnObjectBySTL(void){
	std::cout << "fXgNgDrawnObjectBySTL\n";
}


void DrawnObjectBySTL::SetColor(double r,double g,double b){
	this->singleColor[0] = r;
	this->singleColor[1] = g;
	this->singleColor[2] = b;

}
const float* DrawnObjectBySTL::GetColor(void){
	return this->singleColor;
}



void DrawnObjectBySTL::Draw(void){
	unsigned int i;
	Triangle *t;

	glPushMatrix();

	this->transform->Execute();

	if(this->isSingleColorEnable){
		glMaterialfv(GL_FRONT, GL_DIFFUSE, this->singleColor);
		for(i=0;i<this->triangles.size();i++){
			t = this->triangles[i];
			glNormal3dv(t->normal);
			glBegin(GL_POLYGON);
			glVertex3dv(&(t->v[0][0]));
			glVertex3dv(&(t->v[1][0]));
			glVertex3dv(&(t->v[2][0]));
			glEnd();
		}
	}else{
		for(i=0;i<this->triangles.size();i++){
			t = this->triangles[i];
			glMaterialfv(GL_FRONT, GL_DIFFUSE, t->color);
			glNormal3dv(t->normal);
			glBegin(GL_POLYGON);
			glVertex3dv(&(t->v[0][0]));
			glVertex3dv(&(t->v[1][0]));
			glVertex3dv(&(t->v[2][0]));
			glEnd();
		}
	}

	glPopMatrix();
}

void DrawnObjectBySTL::Dump(int depth,std::stringstream &ret){

	std::string padding = MyUtility::PaddingChar(depth,' ');
	int i = 1;

	ret << padding << "TYPE = STL\n";

	ret << padding << "NAME = " << this->name << std::endl;

	ret << padding << "File path = " << this->filePath << std::endl;

	ret << padding << "Color = " << this->singleColor[0] << ',' << this->singleColor[1] << ',' << this->singleColor[2] << std::endl;

	ret << padding << "Number of Triangles = " << this->triangles.size() << std::endl;

	ret << padding << "Dump Transform" << std::endl;

	this->transform->Dump(depth+2,ret);
}


void DrawnObjectBySTL::Dump(void){
	std::stringstream ret;
	this->Dump(0,ret);

	std::cout << ret.str();

}


bool DrawnObjectBySTL::Load(std::string sourceFileName ){
	FILE *source;
	char line[100];
	char *tok[20];
	char workArea[100];
	int elements;
	int indexOfTriangle = 0;
	int indexOfVertex = 0;

	Triangle *t=new Triangle();
	//Triangle *t;
	source = fopen(sourceFileName.c_str(),"r");
	if(source == NULL){
		return false;
	}

	while(fgets(line,sizeof(line),source)){
		elements = MyUtility::Split(line,' ',tok,workArea);
		//elements = Split(line,' ',tok,workArea);
		if(elements != 0){
			if(strcmp(tok[0],"solid") == 0){
				this->name = tok[1];
			}else if(strcmp(tok[0],"facet") == 0){	//@xNg
				indexOfVertex = 0;
				sscanf(tok[2],"%lf",&t->normal[0]);
				sscanf(tok[3],"%lf",&t->normal[1]);
				sscanf(tok[4],"%lf",&t->normal[2]);
			}else if(strncmp(tok[0],"endfacet",strlen("endfacet")) == 0){
				indexOfTriangle++;
				this->triangles.push_back(t);		//f[^i[
				t = new Triangle();
			}else if(strcmp(tok[0],"vertex") == 0){
				sscanf(tok[1],"%lf",&t->v[indexOfVertex][0]);
				sscanf(tok[2],"%lf",&t->v[indexOfVertex][1]);
				sscanf(tok[3],"%lf",&t->v[indexOfVertex][2]);

				indexOfVertex++;
			}
		}
	}

	fclose(source);

	this->filePath = sourceFileName;
	return true;

}